import { Checkbox, Divider, Radio, Space, Spin } from "antd";
import { categories, levels } from "../mockData";
import { TaxonomySlug, TermDto } from "@nice/common";

import { useEffect, useMemo } from "react";
import { api } from "@nice/client";
import { useSearchParams } from "react-router-dom";
import TermTree from "@web/src/components/models/term/term-tree";
import TermSelect from "@web/src/components/models/term/term-select";

interface FilterSectionProps {
	selectedCategory: string;
	selectedLevel: string;
	onCategoryChange: (category: string) => void;
	onLevelChange: (level: string) => void;
}

interface GetTaxonomyProps {
	categories: string[];
	isLoading: boolean;
}

function useGetTaxonomy({ type }): GetTaxonomyProps {
	const { data, isLoading }: { data: TermDto[]; isLoading: boolean } =
		api.term.findMany.useQuery({
			where: {
				taxonomy: {
					//TaxonomySlug.CATEGORY
					slug: type,
				},
			},
			include: {
				children: true,
			},
			take: 10, // 只取前10个
			orderBy: {
				createdAt: "desc", // 按创建时间降序排列
			},
		});
	const categories = useMemo(() => {
		const allCategories = isLoading
			? []
			: data?.map((course) => course.name);
		return [...Array.from(new Set(allCategories))];
	}, [data]);
	return { categories, isLoading };
}

export default function FilterSection({
	selectedCategory,
	selectedLevel,
	onCategoryChange,
	onLevelChange,
}: FilterSectionProps) {
	const gateGory: GetTaxonomyProps = useGetTaxonomy({
		type: TaxonomySlug.CATEGORY,
	});
	const levels: GetTaxonomyProps = useGetTaxonomy({
		type: TaxonomySlug.LEVEL,
	});

	const [searchParams, setSearchParams] = useSearchParams();
	useEffect(() => {
		if (searchParams.get("category"))
			onCategoryChange(searchParams.get("category"));
	}, [searchParams.get("category")]);

	const { data: taxonomies } = api.taxonomy.getAll.useQuery({});

	return (
		<div className="bg-white p-6 rounded-lg shadow-sm space-y-6">
      <Form>
        
      </Form>
			{taxonomies.map((tax) => {
				return (
					<>
						<div>
							<h3 className="text-lg font-medium mb-4">
								{tax?.name}
							</h3>
							<TermSelect
								multiple
								taxonomyId={tax?.id}></TermSelect>
						</div>
					</>
				);
			})}

			<Divider className="my-6" />

			<div>
				<h3 className="text-lg font-medium mb-4">难度等级</h3>
				<Radio.Group
					value={selectedLevel}
					onChange={(e) => onLevelChange(e.target.value)}
					className="flex flex-col space-y-3">
					{levels.isLoading ? (
						<Spin />
					) : (
						<>
							<Radio value="">全部难度</Radio>
							{levels.categories.map((level) => (
								<Radio key={level} value={level}>
									{level}
								</Radio>
							))}
						</>
					)}
				</Radio.Group>
			</div>
		</div>
	);
}
